library value_broadcast;

class ValueBroadcast<T>{
  final Set<void Function(T)> _listeners = Set();
  final Set<void Function()> _noArgListeners = Set();

  ///High level functions.
  void addListener(final void Function(T) listener) => _listeners.add(listener);
  void removeListener(final void Function(T) listener) => _listeners.remove(listener);

  void addNoArgListener(final void Function() listener) => _noArgListeners.add(listener);
  void removeNoArgListener(final void Function() listener) => _noArgListeners.remove(listener);

  ValueBroadcast(this._value);

  ///Set to private in order to override setter and getter.
  T _value;

  T get value => _value;

  set value(final T update){
    if(_value != update){
      _value = update;
      _listeners.forEach((final void Function(T) listener) => listener(update));
      _noArgListeners.forEach((final void Function() listener) => listener());
    }
  }

  void dispose(){
    _listeners.clear();
    _noArgListeners.clear();
  }
}